@@ -1,4 +1,4 @@
-;# $Id: macros.pl 1 2006-08-24 13:24:12Z rmanfredi $
+;# $Id: macros.pl 46 2008-06-25 16:35:05Z rmanfredi $
 ;#
 ;#  Copyright (c) 1990-2006, Raphael Manfredi
 ;#  
@@ -36,7 +36,7 @@
 ;# %D     Day of the week (0-6)
 ;# %H     Host name (name of the machine on which the mailagent runs)
 ;# %I     Internet domain from sender (domain.ct in user@host.domain.ct)
-;# %L     Length of the message in bytes (without header)
+;# %L     Length of the message in bytes (without header, no transfer encoding)
 ;# %N     Full name of sender (login name if none)
 ;# %O     Organization name from sender address (domain in user@host.domain.ct)
 ;# %R     Subject of orginal message with leading Re: suppressed
